#pragma comment(lib, "dxguid.lib")
#pragma comment(lib, "d3d10.lib")
#pragma comment(lib, "d3dx10.lib")

#include <windows.h>
#include <tchar.h>
#include <d3d10.h>
#include <d3dx10.h>
#include <map>
#include "oxcp.h"
#include "stlparser.h"
#include "oxnumericcounter.h"


using namespace OX;

int cameraCurRot_g = 0;
int wheelCount_g = 0;
int wKeyCount_g = 0;
int sKeyCount_g = 0;
int aKeyCount_g = 0;
int dKeyCount_g = 0;
int qKeyCount_g = 0;

// STL̒_}[W@ω
void merge(
	const std::vector< STLParser::Vertex > &vtx,
	const std::vector< uint32_t > &idx,
	std::vector< STLParser::Vertex > &newVtx,
	std::vector< uint32_t > &newIdx
);


LRESULT CALLBACK WndProc( HWND hWnd, UINT mes, WPARAM wParam, LPARAM lParam ) {

	static int cameraRotStartVal = 0;

	switch ( mes ) {
	case WM_DESTROY:
		PostQuitMessage( 0 );
		return 0;
	case WM_LBUTTONDOWN:
		cameraRotStartVal = LOWORD( lParam );
		break;
	case WM_MOUSEWHEEL: {
		bool bShift = GET_KEYSTATE_WPARAM( wParam ) & MK_SHIFT;
		wheelCount_g += GET_WHEEL_DELTA_WPARAM( wParam ) / WHEEL_DELTA;
		break;
	}
	case WM_MOUSEMOVE:
		if ( wParam & MK_LBUTTON ) {
			int cameraRotDif = LOWORD( lParam ) - cameraRotStartVal;
			cameraRotStartVal = LOWORD( lParam );
			cameraCurRot_g += cameraRotDif;
		}
		break;
	case WM_CHAR:
		if ( wParam == 'w' )
			wKeyCount_g = 1;
		else if ( wParam == 's' )
			sKeyCount_g = 1;
		else if ( wParam == 'a' )
			aKeyCount_g = 1;
		else if ( wParam == 'd' )
			dKeyCount_g = 1;
		else if ( wParam == 'q' )
			qKeyCount_g = 1;
		break;
	}
	return DefWindowProc( hWnd, mes, wParam, lParam );
}

int APIENTRY _tWinMain( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow )
{
	// STLǂݍ
	std::string parseFileName;
	STLParser parser;
	if ( strlen( lpCmdLine ) > 0 ) {
		parseFileName = lpCmdLine;
	}
	else {
		parseFileName = "Saba2.stl";
	}
	if ( parser.parseFromFile( parseFileName.c_str(), true, true ) == false ) {
		MessageBoxA( 0, "STLt@C܂łB", "STLt@CG[", 0 );
		return -1;
	}
	AABB aabb = parser.calcAABB();
	auto vertices = parser.getVertices();
	auto verticesSize = parser.getVerticesSize();
	auto indices = parser.getIndices();

	// STL̒_}[W@𕽋ω
	{
		std::vector< STLParser::Vertex > newVtx;
		std::vector< uint32_t > newIdx;
		merge( vertices, indices, newVtx, newIdx );
		vertices = newVtx;
		indices = newIdx;
		verticesSize = newVtx.size() * sizeof( STLParser::Vertex );
	}

	// AvP[V̏
	TCHAR gName[ 100 ] = _T( "}`_[^[QbgTvvO" );
	MSG msg; HWND hWnd;
	WNDCLASSEX wcex = { sizeof( WNDCLASSEX ), CS_HREDRAW | CS_VREDRAW, WndProc, 0, 0, hInstance, NULL, NULL,
		(HBRUSH)( COLOR_WINDOW + 1 ), NULL, (TCHAR*)gName, NULL };
	if ( !RegisterClassEx( &wcex ) )
		return 0;

	int w = 800, h = 800;
	RECT clientRect = { 0, 0, w, h };
	::AdjustWindowRect( &clientRect, WS_OVERLAPPEDWINDOW, FALSE );

	if ( !( hWnd = CreateWindow( gName, gName, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0,
		clientRect.right - clientRect.left, clientRect.bottom - clientRect.top,
		NULL, NULL, hInstance, NULL ) ) )
		return 0;

	ShowWindow( hWnd, nCmdShow );


	// Direct3D10̏ //
	cp< ID3D10Device > cpDevice_;
	cp< IDXGISwapChain > cpSwapChain_;
	cp< ID3D10RenderTargetView > cpRenderTargetView_;

	// Xbv`FCƃfoCX̏
	{
		DXGI_SWAP_CHAIN_DESC desc;
		memset( &desc, 0, sizeof( desc ) );
		desc.BufferCount = 1;
		desc.BufferDesc.Width = w;
		desc.BufferDesc.Height = h;
		desc.BufferDesc.Format = DXGI_FORMAT_R8G8B8A8_UNORM;
		desc.BufferDesc.Format = DXGI_FORMAT_R16G16B16A16_FLOAT;
		desc.BufferDesc.Scaling = DXGI_MODE_SCALING_UNSPECIFIED;
		desc.BufferDesc.RefreshRate.Numerator = 60;
		desc.BufferDesc.RefreshRate.Denominator = 1;
		desc.BufferDesc.ScanlineOrdering = DXGI_MODE_SCANLINE_ORDER_UNSPECIFIED;
		desc.BufferUsage = DXGI_USAGE_RENDER_TARGET_OUTPUT;
		desc.Flags = DXGI_SWAP_CHAIN_FLAG_ALLOW_MODE_SWITCH;
		desc.OutputWindow = hWnd;
		desc.SampleDesc.Count = 1;
		desc.SampleDesc.Quality = 0;
		desc.SwapEffect = DXGI_SWAP_EFFECT_DISCARD;
		desc.Windowed = TRUE;

		if ( FAILED( D3D10CreateDeviceAndSwapChain(
			0,
			D3D10_DRIVER_TYPE_HARDWARE,
			0,
			0,
			D3D10_SDK_VERSION,
			&desc,
			cpSwapChain_.toCreator(),
			cpDevice_.toCreator()
		) ) ) {
			return false;
		}
	}

	// obNobt@m
	ID3D10Texture2D *backBuffer;
	if ( FAILED( cpSwapChain_->GetBuffer( 0, __uuidof( ID3D10Texture2D ), (void**)&backBuffer ) ) ) {
		return false;
	}

	cp< ID3D10RenderTargetView > backBufferRenderTargetView_;
	if ( FAILED( cpDevice_->CreateRenderTargetView( backBuffer, 0, backBufferRenderTargetView_.toCreator() ) ) ) {
		return false;
	}
	cpDevice_->OMSetRenderTargets( 1, backBufferRenderTargetView_.getPtrPtr(), 0 );	// o̓}[Wɐݒ

	// [xXeVobt@쐬
	cp< ID3D10Texture2D > depthStencil;
	D3D10_TEXTURE2D_DESC dstDesc;
	dstDesc.Width = w;
	dstDesc.Height = h;
	dstDesc.MipLevels = 1;
	dstDesc.ArraySize = 1;
	dstDesc.Format = DXGI_FORMAT_D32_FLOAT;
	dstDesc.SampleDesc.Count = 1;
	dstDesc.SampleDesc.Quality = 0;
	dstDesc.Usage = D3D10_USAGE_DEFAULT;
	dstDesc.BindFlags = D3D10_BIND_DEPTH_STENCIL;
	dstDesc.CPUAccessFlags = 0;
	dstDesc.MiscFlags = 0;
	cpDevice_->CreateTexture2D( &dstDesc, 0, depthStencil.toCreator() );

	cp< ID3D10DepthStencilView > dsView;
	D3D10_DEPTH_STENCIL_VIEW_DESC dsvDesc;
	dsvDesc.Format = dstDesc.Format;
	dsvDesc.ViewDimension = D3D10_DSV_DIMENSION_TEXTURE2D;
	dsvDesc.Texture2D.MipSlice = 0;
	cpDevice_->CreateDepthStencilView( depthStencil.getPtr(), &dsvDesc, dsView.toCreator() );

	// r[|[g
	D3D10_VIEWPORT vp;
	vp.TopLeftX = 0;
	vp.TopLeftY = 0;
	vp.Width = w;
	vp.Height = h;
	vp.MinDepth = 0.0f;
	vp.MaxDepth = 1.0f;

	// _obt@
	D3D10_BUFFER_DESC vtxBufDesc;
	vtxBufDesc.Usage = D3D10_USAGE_DEFAULT;
	vtxBufDesc.ByteWidth = verticesSize;
	vtxBufDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	vtxBufDesc.CPUAccessFlags = 0;
	vtxBufDesc.MiscFlags = 0;

	D3D10_SUBRESOURCE_DATA vtxData;
	memset( &vtxData, 0, sizeof( vtxData ) );
	vtxData.pSysMem = &vertices[ 0 ];

	cp< ID3D10Buffer > vtxBuffer;
	cpDevice_->CreateBuffer( &vtxBufDesc, &vtxData, vtxBuffer.toCreator() );

	// CfbNXobt@
	D3D10_BUFFER_DESC idxBufDesc;
	memset( &idxBufDesc, 0, sizeof( idxBufDesc ) );
	idxBufDesc.Usage = D3D10_USAGE_DEFAULT;
	idxBufDesc.ByteWidth = indices.size() * sizeof( uint32_t );
	idxBufDesc.BindFlags = D3D10_BIND_INDEX_BUFFER;

	D3D10_SUBRESOURCE_DATA idxData;
	memset( &idxData, 0, sizeof( idxData ) );
	idxData.pSysMem = &indices[ 0 ];

	cp< ID3D10Buffer > idxBuffer;
	cpDevice_->CreateBuffer( &idxBufDesc, &idxData, idxBuffer.toCreator() );

	// GtFNg
	cp< ID3D10Effect > effect;
	ID3D10Blob *err = 0;
	auto res = D3DX10CreateEffectFromFile( "multirendertarget.fx", 0, 0, "fx_4_0", D3D10_SHADER_ENABLE_STRICTNESS, 0, cpDevice_.getPtr(), 0, 0, effect.toCreator(), &err, 0 );;
	if ( FAILED( res ) ) {
		const char* errStr = (const char*)err->GetBufferPointer();
		OutputDebugStringA( errStr );
		return 0;
	}

	// ̓CAEg
	D3D10_INPUT_ELEMENT_DESC layout[] = {
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
	{ "TEXCOORD", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 }
	};

	cp< ID3D10InputLayout > vtxLayout;
	ID3D10EffectTechnique *tech = effect->GetTechniqueByName( "tech" );
	ID3D10EffectPass *pass = tech->GetPassByName( "pass_MRT" );
	D3D10_PASS_DESC passDesc;
	pass->GetDesc( &passDesc );
	cpDevice_->CreateInputLayout( layout, 2, passDesc.pIAInputSignature, passDesc.IAInputSignatureSize, vtxLayout.toCreator() );


	// MRT֘Ȁ //

	// _[^[Qbg𕡐쐬_[^[Qbgr[ɓo^
	D3D10_TEXTURE2D_DESC rtDesc;
	memset( &rtDesc, 0, sizeof( rtDesc ) );
	rtDesc.Width = w;
	rtDesc.Height = h;
	rtDesc.MipLevels = 1;
	rtDesc.Format = DXGI_FORMAT_R16G16B16A16_FLOAT;
	rtDesc.SampleDesc.Count = 1;
	rtDesc.Usage = D3D10_USAGE_DEFAULT;
	rtDesc.ArraySize = 1;
	rtDesc.BindFlags = D3D10_BIND_RENDER_TARGET | D3D10_BIND_SHADER_RESOURCE;
	rtDesc.CPUAccessFlags = 0;

	const int mrtNum = 2;
	cp< ID3D10Texture2D > cpRenderTexes[ mrtNum ] = {};
	cp< ID3D10RenderTargetView > renderTargetViewStock[ mrtNum ] = {};
	cp< ID3D10ShaderResourceView > cpShaderResViews[ mrtNum ];
	for ( int i = 0; i < mrtNum; ++i ) {
		cpDevice_->CreateTexture2D( &rtDesc, 0, cpRenderTexes[ i ].toCreator() );
		cpDevice_->CreateRenderTargetView( cpRenderTexes[ i ].getPtr(), 0, renderTargetViewStock[ i ].toCreator() );

		// _[^[Qbg̓VF[_\[XƂĂgp
		cpDevice_->CreateShaderResourceView( cpRenderTexes[ i ].getPtr(), 0, cpShaderResViews[ i ].toCreator() );
	}

	// |XgGtFNgp̃XN[|S
	struct ScVtx {
		float x, y, z;
		float u, v;
	} scVtx[ 4 ] = {
		{ -1.0f, -1.0f, 0.0f,  0.0f, 1.0f },
		{ -1.0f,  1.0f, 0.0f,  0.0f, 0.0f },
		{  1.0f, -1.0f, 0.0f,  1.0f, 1.0f },
		{  1.0f,  1.0f, 0.0f,  1.0f, 0.0f },
	};
	D3D10_BUFFER_DESC screenVtxBufDesc;
	screenVtxBufDesc.Usage = D3D10_USAGE_DEFAULT;
	screenVtxBufDesc.ByteWidth = sizeof( scVtx );
	screenVtxBufDesc.BindFlags = D3D10_BIND_VERTEX_BUFFER;
	screenVtxBufDesc.CPUAccessFlags = 0;
	screenVtxBufDesc.MiscFlags = 0;

	D3D10_SUBRESOURCE_DATA screenVtxData;
	memset( &screenVtxData, 0, sizeof( screenVtxData ) );
	screenVtxData.pSysMem = scVtx;

	cp< ID3D10Buffer > screenVtxBuffer;
	cpDevice_->CreateBuffer( &screenVtxBufDesc, &screenVtxData, screenVtxBuffer.toCreator() );

	// XN[|S̓̓CAEg
	D3D10_INPUT_ELEMENT_DESC screenLayout[] = {
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D10_INPUT_PER_VERTEX_DATA, 0 },
		{ "TEXCOORD", 0, DXGI_FORMAT_R32G32_FLOAT, 0, 12, D3D10_INPUT_PER_VERTEX_DATA, 0 }
	};

	cp< ID3D10InputLayout > screenVtxLayout;
	ID3D10EffectPass *scPass = tech->GetPassByName( "pass_AxisBlur" );
	D3D10_PASS_DESC scPassDesc;
	scPass->GetDesc( &scPassDesc );
	cpDevice_->CreateInputLayout( screenLayout, 2, scPassDesc.pIAInputSignature, scPassDesc.IAInputSignatureSize, screenVtxLayout.toCreator() );


	// `掞p[^ //

	// IuWFNg̒__ɈړAX-90x]
	D3DXMATRIX localOfsMat, rotXMat, offsetCenterMat, placeXZMat;
	float rotX = D3DXToRadian( -90.0f );
	D3DXMatrixRotationX( &rotXMat, rotX );
	AABB worldAABB = aabb.rot( AABB::Axis_X );
	D3DXMatrixTranslation( &offsetCenterMat, -aabb.center_.x, -aabb.center_.y, -aabb.center_.z );
	localOfsMat = offsetCenterMat * rotXMat;

	// ep[^
	float fovY = D3DXToRadian( 30.0f );
	float cameraDist = worldAABB.halfLen3_.len() / sinf( fovY / 2 );
	NumericCounter cameraRotCtr( 0.0f, 0.001f, 0.075f );
	NumericCounter zoomCtr( 0.0f, 0.001f, 0.075f );
	float zoomUnit = cameraDist * 0.05f;
	NumericCounter modelAlphaCtr( 1.0f, 0.001f, 0.15f, 0.0f, 1.0f );
	float curModelAlpha = 1.0f;
	NumericCounter specPowerCtr( 2.0f, 0.001f, 0.1f, 0.0f, 100.0f );
	float specPowerUnit = 2.0f;
	NumericCounter specEffCtr( 0.5f, 0.001f, 0.1f, 0.0f, 1.0f );
	float specEffUnit = 0.1f;
	Vec4 blurUVUnitX( 1.0f / w, 0.0f, 0.0f, 0.0f );
	Vec4 blurUVUnitY( 0.0f, 1.0f / h, 0.0f, 0.0f );
	int blurUVCount = 40;
	float brightnessPower = 10.0f;

	D3DXVECTOR4 clearColors[ mrtNum + 1 ] = {
		D3DXVECTOR4( 0.2f, 0.2f, 0.2f, 1.0f ),	// obNobt@
		D3DXVECTOR4( 0.0f, 0.0f, 0.0f, 1.0f ),	// RT0
		D3DXVECTOR4( 0.0f, 0.0f, 0.0f, 1.0f )	// RT1
	};

	// bZ[W [v
	do {
		if ( PeekMessage( &msg, NULL, 0, 0, PM_REMOVE ) ) {
			TranslateMessage( &msg );
			DispatchMessage( &msg );
		}
		else {
			// p[^XV
			cameraRotCtr.setAim( -cameraCurRot_g * 0.01f );
			auto cameraRot = cameraRotCtr.update();
			zoomCtr.setAim( zoomCtr.getAim() + wheelCount_g * zoomUnit );
			auto zoomDist = zoomCtr.update();
			specPowerCtr.setAim( specPowerCtr.getAim() + ( wKeyCount_g - sKeyCount_g ) * specPowerUnit );
			specEffCtr.setAim( specEffCtr.getAim() + ( aKeyCount_g - dKeyCount_g ) * specEffUnit );
			if ( qKeyCount_g != 0 ) {
				curModelAlpha = ( curModelAlpha > 0.0f ? 0.0f : 1.0f );
				modelAlphaCtr.setAim( curModelAlpha );
			}
			wheelCount_g = 0;
			wKeyCount_g = 0;
			sKeyCount_g = 0;
			aKeyCount_g = 0;
			dKeyCount_g = 0;
			qKeyCount_g = 0;

			// esݒ
			D3DXMATRIX world, view, proj;
			D3DXMatrixRotationY( &world, cameraRot );
			world = localOfsMat * world;
			D3DXMatrixLookAtLH( &view, &D3DXVECTOR3( 0.0f, 0.0f, -cameraDist + zoomDist ), &D3DXVECTOR3( 0.0f, 0.0f, 0.0f ), &D3DXVECTOR3( 0.0f, 1.0f, 0.0f ) );
			D3DXMatrixPerspectiveFovLH( &proj, D3DXToRadian( 30.0f ), (float)w/h, 0.1f, cameraDist * 2.0f );

			ID3D10RenderTargetView *renderTargetViews[ mrtNum + 1 ] = {
				backBufferRenderTargetView_.getPtr(),
				renderTargetViewStock[ 0 ].getPtr(),
				renderTargetViewStock[ 1 ].getPtr()
			};

			cpDevice_->RSSetViewports( 1, &vp );
			for ( int i = 0; i < mrtNum + 1; ++i )
				cpDevice_->ClearRenderTargetView( renderTargetViews[ i ], clearColors[ i ] );
			cpDevice_->ClearDepthStencilView( dsView.getPtr(), D3D10_CLEAR_DEPTH, 1.0f, 0 );


			// @ MRTŃfBt[YEXyL`ƋPx}bv𓯎ɕ`
			cpDevice_->OMSetRenderTargets( 2, renderTargetViews, dsView.getPtr() );		// RT2iobNobt@ART0jݒ
			UINT stride = sizeof( STLParser::Vertex );
			UINT offset = 0;
			cpDevice_->IASetVertexBuffers( 0, 1, vtxBuffer.getPtrPtr(), &stride, &offset );
			cpDevice_->IASetIndexBuffer( idxBuffer.getPtr(), DXGI_FORMAT_R32_UINT, 0 );
			cpDevice_->IASetInputLayout( vtxLayout.getPtr() );
			cpDevice_->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLELIST );

			D3DXMATRIX wv = world * view;
			D3DXMATRIX wvp = wv * proj;
			auto worldV = effect->GetVariableByName( "wv" );
			worldV->AsMatrix()->SetMatrix( (float*)&wv );
			auto wvpV = effect->GetVariableByName( "wvp" );
			wvpV->AsMatrix()->SetMatrix( (float*)&wvp );
			auto specPowerV = effect->GetVariableByName( "specPower" );
			specPowerV->AsScalar()->SetFloat( specPowerCtr.update() );
			auto specEffV = effect->GetVariableByName( "specEff" );
			specEffV->AsScalar()->SetFloat( specEffCtr.update() );
			auto modelAlphaV = effect->GetVariableByName( "modelAlpha" );
			modelAlphaV->AsScalar()->SetFloat( modelAlphaCtr.update() );

			tech->GetPassByName( "pass_MRT" )->Apply( 0 );
			cpDevice_->DrawIndexed( indices.size(), 0, 0 );

			// A 쐬ꂽPx}bvɂȂ񂿂ăKEXu[Kp
			// X
			stride = sizeof( ScVtx );
			cpDevice_->IASetVertexBuffers( 0, 1, screenVtxBuffer.getPtrPtr(), &stride, &offset );
			cpDevice_->IASetInputLayout( screenVtxLayout.getPtr() );
			cpDevice_->IASetPrimitiveTopology( D3D10_PRIMITIVE_TOPOLOGY_TRIANGLESTRIP );

			auto blurUVCountV = effect->GetVariableByName( "blurUVCount" );
			blurUVCountV->AsScalar()->SetInt( blurUVCount );
			auto blurUVUnitV = effect->GetVariableByName( "blurUVUnit" );
			blurUVUnitV->AsVector()->SetFloatVector( blurUVUnitX.f );
			auto texV = effect->GetVariableByName( "tex" );
			texV->AsShaderResource()->SetResource( cpShaderResViews[ 0 ].getPtr() );	// ͂RT0
			cpDevice_->OMSetRenderTargets( 1, &renderTargetViews[ 2 ], 0 );				// o͂RT1

			tech->GetPassByName( "pass_AxisBlur" )->Apply( 0 );
			cpDevice_->Draw( 4, 0 );

			// Y
			blurUVUnitV->AsVector()->SetFloatVector( blurUVUnitY.f );
			texV->AsShaderResource()->SetResource( cpShaderResViews[ 1 ].getPtr() );	// ͂RT1
			cpDevice_->OMSetRenderTargets( 1, &renderTargetViews[ 1 ], 0 );				// o͂RT0

			tech->GetPassByName( "pass_AxisBlur" )->Apply( 0 );
			cpDevice_->Draw( 4, 0 );

			// B Pxڂ摜
			auto brightnessPowerV = effect->GetVariableByName( "brightnessPower" );
			brightnessPowerV->AsScalar()->SetFloat( brightnessPower );
			texV->AsShaderResource()->SetResource( cpShaderResViews[ 0 ].getPtr() );		// ͂RT0
			cpDevice_->OMSetRenderTargets( 1, backBufferRenderTargetView_.getPtrPtr(), 0 );	// o͂̓obNobt@

			tech->GetPassByName( "pass_Add" )->Apply( 0 );
			cpDevice_->Draw( 4, 0 );

			// Xbv
			cpSwapChain_->Present( 1, 0 );
		}
	} while ( msg.message != WM_QUIT );

	cpDevice_->ClearState();

	return 0;
}

// }[W֘A //
struct VtxNumber {
	union {
		int64_t number;
		struct {
			int16_t x;
			int16_t y;
			int16_t z;
		};
	};
	VtxNumber() {}
	VtxNumber( const OX::Vec3 &v ) : x( (int16_t)v.x ), y( (int16_t)v.y ), z( (int16_t)v.z ) {}
};

struct VtxGroup {
	OX::STLParser::Vertex v;
	int32_t idx;
	VtxGroup() {}
	VtxGroup( const OX::STLParser::Vertex &v, uint32_t idx ) : v( v ), idx( idx ) {}
};

// _}[W
void merge(
	const std::vector< STLParser::Vertex > &vtx, 
	const std::vector< uint32_t > &idx,
	std::vector< STLParser::Vertex > &newVtx,
	std::vector< uint32_t > &newIdx
) {

	std::map< int64_t, std::vector< VtxGroup > > compGroup;	// rO[v
	std::map< uint32_t, std::vector< STLParser::Vertex > > mergeGroup;	// }[WO[v

	uint32_t vtxCounter = 0;
	for ( size_t i = 0; i < idx.size(); ++i ) {

		uint32_t e = idx[ i ];
		const OX::STLParser::Vertex &v = vtx[ e ];

		VtxNumber vn( v.p_ );	// W𐔒lɕϊ

		if ( compGroup.find( vn.number ) == compGroup.end() ) {
			// VKrO[vo^
			std::vector< VtxGroup > newGroup;
			newGroup.push_back( VtxGroup( v, vtxCounter ) );
			compGroup[ vn.number ] = newGroup;

			// VK}[WO[vo^
			std::vector< STLParser::Vertex > newMergeG;
			newMergeG.push_back( v );
			mergeGroup[ vtxCounter ] = newMergeG;

			// _obt@ƃCfbNXobt@ɒǉ
			newVtx.push_back( v );
			newIdx.push_back( vtxCounter );

			vtxCounter++;
		}
		else {
			// rO[vœW`FbN
			bool bDetectMerge = false;
			auto &group = compGroup[ vn.number ];
			for ( size_t j = 0; j < group.size(); ++j ) {
				if ( ( group[ j ].v.p_ - vtx[ e ].p_ ).lenSq() < 0.00001f ) {
					// }[WWƂ݂Ȃ
					size_t index = group[ j ].idx;
					mergeGroup[ index ].push_back( vtx[ e ] );	// mergeGroupmergeIdx݂͑Ă͂

					// _̓}[Wꂽ̂ŃCfbNX̂ݍXVB_JE^͒ǉȂB
					newIdx.push_back( index );
					bDetectMerge = true;

					break;
				}
			}
			if ( bDetectMerge == false ) {
				// }[W_̂ŔrO[vɒǉ
				group.push_back( VtxGroup( v, vtxCounter ) );
				mergeGroup[ vtxCounter ].push_back( vtx[ e ] );
				// _obt@ƃCfbNXobt@ɒǉ
				newVtx.push_back( v );
				newIdx.push_back( vtxCounter );
				
				vtxCounter++;
			}
			else {
			}
		}
	}

	// }[WO[vŖ@𕽋ω
	auto it = mergeGroup.begin();
	for ( ; it != mergeGroup.end(); ++it ) {
		OX::Vec3 aveN;
		auto &vs = it->second;
		for ( size_t i = 0; i < vs.size(); ++i ) {
			aveN += vs[ i ].n_;
		}
		aveN /= ( float )vs.size();
		newVtx[ it->first ].n_ = aveN.normND();
	}
}
