matrix wv;
matrix wvp;
float modelAlpha;
float specPower;
float specEff;
int blurUVCount;
float2 blurUVUnit;
float brightnessPower;

struct VS_INPUT {
	float3 pos : POSITION;
	float3 norm : TEXCOORD0;
};

struct PS_INPUT {
	float4 pos : SV_POSITION;
	float3 norm : TEXCOORD0;
	float3 posV : TEXCOORD1;
};

struct VS_INPUT_POSTEFF {
	float3 pos : POSITION;
	float2 uv : TEXCOORD0;
};

struct PS_INPUT_POSTEFF {
	float4 pos : SV_POSITION;
	float2 uv : TEXCOORD0;
};

struct PS_OUTPUT {
	float4 target0 : SV_TARGET0;
	float4 target1 : SV_TARGET1;
	float4 target2 : SV_TARGET2;
	float4 target3 : SV_TARGET3;
};

Texture2D tex;

SamplerState smp {
	Filter = MIN_MAG_MIP_POINT;
	AddressU = Border;
	AddressV = Border;
};


PS_INPUT vsMain( VS_INPUT In ) {
	PS_INPUT Out;
	Out.pos = mul( float4( In.pos, 1.0 ), wvp );
	Out.norm = mul( float4( In.norm, 0.0 ), wv ).xyz;
	Out.posV = mul( float4( In.pos, 1.0 ), wv ).xyz;
	return Out;
}

PS_OUTPUT psMain( PS_INPUT In ) {
	PS_OUTPUT Out;

	// fBt[Y
	float3 lightDirV = float3( 0.3, -0.2, 1.0 );
	lightDirV = normalize( lightDirV );
	float3 n = normalize( In.norm );
	float diffusePower = clamp( dot( -lightDirV, n ), 0.0, 1.0 );
	float3 diffuseColor = float3( 0.5, 0.5, 0.5 ) * diffusePower;

	// XyL
	float3 refVec = reflect( lightDirV, n );
	float specVal = pow( clamp( -dot( normalize( In.posV ), refVec ), 0.00001, 1.0 ), specPower );

	float3 speculerColor = float3( 0.5, 0.5, 0.5 ) * specVal * specEff;
	float3 color = diffuseColor + speculerColor;

	Out.target0 = float4( color, modelAlpha );

	// PxvZ
	// J[̒1𒴂Ă獂PxŋPĂƂ݂Ȃ
	float brightness = smoothstep( 1.0, 2.0, length( color ) );
	Out.target1 = float4( brightness, brightness, brightness, 1.0 );

	return Out;
}

// |XgGtFNgVS
PS_INPUT_POSTEFF vsMainPE( VS_INPUT_POSTEFF In ) {
	PS_INPUT_POSTEFF Out;
	Out.pos = float4( In.pos, 1.0 );
	Out.uv = In.uv;
	return Out;
}

// Px}bvɎu[
float4 psMainAxisBlur( PS_INPUT_POSTEFF In ) : SV_TARGET0 {
	float3 color = float3( 0.0, 0.0, 0.0 );
	float weightTotal = 0.0;
	for ( int i = -blurUVCount; i <= blurUVCount; ++i ) {
		float weight = 1.0 - smoothstep( 0.0, blurUVCount + 1, abs( i ) );
		color += tex.Sample( smp, In.uv + blurUVUnit * i ).xyz * weight;
		weightTotal += weight;
	}
	return float4( color / weightTotal, 1.0 );
}


// Px}bv
float4 psMainAdd( PS_INPUT_POSTEFF In ) : SV_TARGET0 {
	// Px}bvobNobt@ɉZ
	return tex.Sample( smp, In.uv ) * brightnessPower;
}

RasterizerState rs {
	CullMode = Back;
};

BlendState bsMRT {
	AlphaToCoverageEnable = FALSE;
	BlendEnable[ 0 ] = TRUE;
	BlendEnable[ 1 ] = TRUE;
	SrcBlend = SRC_ALPHA;
	DestBlend = INV_SRC_ALPHA;
	BlendOp = ADD;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ZERO;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[ 0 ] = 0x0F;
	RenderTargetWriteMask[ 1 ] = 0x0F;
};

DepthStencilState dssEnable {
	DepthEnable = TRUE;
};

BlendState bsAxisBlur {
	AlphaToCoverageEnable = FALSE;
	BlendEnable[ 0 ] = TRUE;
	SrcBlend = ONE;
	DestBlend = ZERO;
	BlendOp = ADD;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ZERO;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[ 0 ] = 0x0F;
};

DepthStencilState dssDisable {
	DepthEnable = FALSE;
};

BlendState bsBlur {
	AlphaToCoverageEnable = FALSE;
	BlendEnable[ 0 ] = TRUE;
	SrcBlend = ONE;
	DestBlend = ONE;
	BlendOp = ADD;
	SrcBlendAlpha = ONE;
	DestBlendAlpha = ZERO;
	BlendOpAlpha = ADD;
	RenderTargetWriteMask[ 0 ] = 0x0F;
};

technique10 tech {
	pass pass_MRT {
		SetVertexShader( CompileShader( vs_4_0, vsMain() ) );
		SetPixelShader( CompileShader( ps_4_0, psMain() ) );
		
		SetRasterizerState( rs );
		SetBlendState( bsMRT, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( dssEnable, 0 );
	}
	pass pass_AxisBlur {
		SetVertexShader( CompileShader( vs_4_0, vsMainPE() ) );
		SetPixelShader( CompileShader( ps_4_0, psMainAxisBlur() ) );

		SetRasterizerState( rs );
		SetBlendState( bsAxisBlur, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( dssDisable, 0 );
	}
	pass pass_Add {
		SetVertexShader( CompileShader( vs_4_0, vsMainPE() ) );
		SetPixelShader( CompileShader( ps_4_0, psMainAdd() ) );

		SetRasterizerState( rs );
		SetBlendState( bsBlur, float4( 0.0f, 0.0f, 0.0f, 0.0f ), 0xFFFFFFFF );
		SetDepthStencilState( dssDisable, 0 );
	}
}
